function w = weightfun(resid,s,wfun,h)
% Funzione peso robusta

% resid sono i residui
% s e' una stima (meglio se robusta) della varianza dei residui, vedi ROBSTD. 
% wfun speficica una funzione peso. Default = 'bisquare'
% h costante che moltiplica la tuning constant standard per quella funzione. Default = 1

% See also: ROBSTD

if nargin < 4
h =1;
end

if nargin < 3
wfun = 'bisquare';
end

% Convert name of weight function to a handle to a local function, and get
% the default value of the tuning parameter
    switch(wfun)
        case 'andrews'
            wfun = @andrews;
            t = 1.339;
        case 'bisquare'
            wfun = @bisquare;
            t = 4.685;
        case 'cauchy'
            wfun = @cauchy;
            t= 2.385;
        case 'fair'
            wfun = @fair;
            t = 1.400;
        case 'huber'
            wfun = @huber;
            t = 1.345;
        case 'logistic'
            wfun = @logistic;
            t = 1.205;
        case 'ols'
            wfun = @ols;
            t = 1;
        case 'talwar'
            wfun = @talwar;
            t = 2.795;
        case 'welsch'
            wfun = @welsch;
            t = 2.985;
               otherwise
            disp('Unknown method.')
    end

     
w = wfun(resid./(t*s*h));

% --------- weight functions

function w = andrews(r)
r = max(sqrt(eps(class(r))), abs(r));
w = (abs(r)<pi) .* sin(r) ./ r;

function w = bisquare(r)
w = (abs(r)<1) .* (1 - r.^2).^2;

function w = cauchy(r)
w = 1 ./ (1 + r.^2);

function w = fair(r)
w = 1 ./ (1 + abs(r));

function w = huber(r)
w = 1 ./ max(1, abs(r));

function w = logistic(r)
r = max(sqrt(eps(class(r))), abs(r));
w = tanh(r) ./ r;

function w = ols(r)
w = ones(size(r));

function w = talwar(r)
w = 1 * (abs(r)<1);

function w = welsch(r)
w = exp(-(r.^2));